/*
 procinfo -- list process information
 Copyright (C) 2007 Daniel Molina Wegener <dmw@coder.cl>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307  US
 */

#define PCRE_STATIC 1

#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <psapi.h>
#include <pcre.h>

#include "pinfo.h"
#include "pinfo_grep.h"
#include "proc_tool.h"


DWORD total_procs = 0;


PROC_LIST_T *
pinfo_getproclist () {
	PROC_LIST_T *pl = (PROC_LIST_T *)NULL;
	pl = (PROC_LIST_T *)malloc (sizeof (PROC_LIST_T));
	if (pl != (PROC_LIST_T *)NULL) {
		pl->list = (DWORD *)malloc (PS_LIST_SZ);
		if (pl->list != (DWORD *)NULL) {
			pl->list_size = PS_LIST_SZ;
			pl->list_count = PS_LIST_COUNT;
			memset (pl->list, 0, PS_LIST_SZ);
			if (FALSE == (EnumProcesses (pl->list, PS_LIST_SZ, &pl->required_count))) {
				pl->list_count = (DWORD)(pl->required_count / sizeof(DWORD));
				fprintf (stderr, "error getting process list\n");
				free (pl->list);
				free (pl);
				pl = (PROC_LIST_T *)NULL;
			} else {
				printf ("total process: %ld\n", pl->required_count);
			}
		}
	}
	return pl;
}


HMODULE_LIST_T *
pinfo_getprocmodlist (HANDLE p) {
	HMODULE_LIST_T *hmr = (HMODULE_LIST_T *)NULL;
	if (p > 0) {
		hmr = (HMODULE_LIST_T *)malloc (sizeof(HMODULE_LIST_T));
		if (hmr != NULL) {
			hmr->list = (HMODULE *)malloc (PM_LIST_SZ);
			if (hmr->list != (HMODULE *)NULL) {
				memset (hmr->list, 0, PM_LIST_SZ);
				hmr->list_sz = 0;
				hmr->list_count = 0;
				hmr->required_sz = 0;
				if ((EnumProcessModules (p, hmr->list, PM_LIST_SZ, &(hmr->required_sz))) == TRUE) {
					if (hmr->required_sz > 0) {
						hmr->list_count = hmr->required_sz / sizeof(HMODULE);
						return hmr;
					}
				}
				free (hmr->list);
			}
			free (hmr);
		}
		hmr = (HMODULE_LIST_T *)NULL;
	}
	return hmr;
}


char *
pinfo_getmodname (HANDLE pid, HMODULE hm) {
	char *hmn = (char *)malloc (512);
	if (hm != NULL) {
		if (hmn != (char *)NULL) {
			memset ((void *)hmn, 0, 512);
			GetModuleFileNameExA(pid, hm, hmn, 512);
			return hmn;
		}
	}
	return NULL;
}


MODULEINFO *
pinfo_getmodinfo (HANDLE pid, HMODULE hm) {
	MODULEINFO *r = (MODULEINFO *)NULL;
	if (pid > 0) {
		r = (MODULEINFO *)malloc (sizeof(MODULEINFO));
		if (r != (MODULEINFO *)NULL) {
			GetModuleInformation(pid, hm, r, sizeof(MODULEINFO));
			return r;
		}
	}
	return r;
}


void
pinfo_longlist (int units) {
	PROC_LIST_T *proc_list = (PROC_LIST_T *)NULL;
	DWORD proc_current;
	DWORD proc_runner;
	HANDLE proc_handle;

	HMODULE_LIST_T *mod_list = (HMODULE_LIST_T *)NULL;
	DWORD mod_runner;
	HMODULE mod_current;
	MODULEINFO *mod_info = (MODULEINFO *)NULL;
	char *mod_name;

	proc_list = pinfo_getproclist ();
	if (proc_list != (PROC_LIST_T *)NULL) {
		for (proc_runner = 0; proc_runner < proc_list->required_count; proc_runner++) {
			proc_current = proc_list->list[proc_runner];
			if (proc_current != 0) {
				proc_handle = OpenProcess (PROCESS_ALL_ACCESS, FALSE, proc_current);
				if (proc_handle != 0) {
					mod_list = pinfo_getprocmodlist (proc_handle);
					if (mod_list != (HMODULE_LIST_T *)NULL) {
						if (mod_list->list_count >= 1) {
							mod_current = mod_list->list[0];
							mod_name = pinfo_getmodname (proc_handle, mod_current);
							mod_info = pinfo_getmodinfo (proc_handle, mod_current);
							if (mod_name != (char *)NULL && mod_info != (MODULEINFO *)NULL) {
								printf ("[%ld] %s\n", proc_current, mod_name);
								printf ("\tBase Addr   : 0x%p\n", (void *)mod_info->lpBaseOfDll);
								printf ("\tEntry Point : 0x%p\n", (void *)mod_info->EntryPoint);
								print_unit (mod_info->SizeOfImage, units, 1);
							}
							if (mod_name != (char *)NULL) {
								free (mod_name);
							}
							if (mod_info != (MODULEINFO *)NULL) {
								free (mod_info);
							}
							printf ("\tModules:\n");
							for (mod_runner = 1; mod_runner < mod_list->list_count; mod_runner++) {
								mod_current = mod_list->list[mod_runner];
								mod_name = pinfo_getmodname (proc_handle, mod_current);
								if (mod_name != (char *)NULL) {
									printf ("\t%s\n", mod_name);
									free (mod_name);
								}
								mod_info = pinfo_getmodinfo (proc_handle, mod_current);
								if (mod_info != (MODULEINFO *)NULL) {
									printf ("\t\tBase Addr   : 0x%p\n", (void *)mod_info->lpBaseOfDll);
									printf ("\t\tEntry Point : 0x%p\n", (void *)mod_info->EntryPoint);
									printf ("\t\tImage Size  : 0x%ld bytes\n", mod_info->SizeOfImage);
									print_unit (mod_info->SizeOfImage, units, 2);
									free (mod_info);
								}
							}
							free (mod_list->list);
							free (mod_list);
						}
					}
				}
				CloseHandle (proc_handle);
			}
		}
		free (proc_list->list);
		free (proc_list);
	}
}


void
pinfo_shortlist (int units) {
	PROC_LIST_T *proc_list = (PROC_LIST_T *)NULL;
	DWORD proc_current;
	DWORD proc_runner;
	HANDLE proc_handle;

	HMODULE_LIST_T *mod_list = (HMODULE_LIST_T *)NULL;
	HMODULE mod_current;
	MODULEINFO *mod_info = (MODULEINFO *)NULL;
	char *mod_name;

	proc_list = pinfo_getproclist ();
	if (proc_list != (PROC_LIST_T *)NULL) {
		for (proc_runner = 0; proc_runner < proc_list->required_count; proc_runner++) {
			proc_current = proc_list->list[proc_runner];
			if (proc_current != 0) {
				proc_handle = OpenProcess (PROCESS_ALL_ACCESS, FALSE, proc_current);
				if (proc_handle != 0) {
					mod_list = pinfo_getprocmodlist (proc_handle);
					if (mod_list != (HMODULE_LIST_T *)NULL) {
						if (mod_list->list_count >= 1) {
							mod_current = mod_list->list[0];
							mod_name = pinfo_getmodname (proc_handle, mod_current);
							mod_info = pinfo_getmodinfo (proc_handle, mod_current);
							if (mod_name != (char *)NULL && mod_info != (MODULEINFO *)NULL) {
								printf ("[%ld] %s\n", proc_current, mod_name);
								printf ("\tBase Addr   : 0x%p\n", (void *)mod_info->lpBaseOfDll);
								printf ("\tEntry Point : 0x%p\n", (void *)mod_info->EntryPoint);
								print_unit (mod_info->SizeOfImage, units, 1);
							}
							if (mod_name != (char *)NULL) {
								free (mod_name);
							}
							if (mod_info != (MODULEINFO *)NULL) {
								free (mod_info);
							}
						}
					}
				}
				CloseHandle (proc_handle);
			}
		}
		free (proc_list->list);
		free (proc_list);
	}

}


void
pinfo_psearch_list (const char *matcher, int longlist, int actionmode, int units) {
	PROC_LIST_T *proc_list = (PROC_LIST_T *)NULL;
	DWORD proc_current;
	DWORD proc_runner;
	HANDLE proc_handle;

	HMODULE_LIST_T *mod_list = (HMODULE_LIST_T *)NULL;
	DWORD mod_runner;
	HMODULE mod_current;
	MODULEINFO *mod_info = (MODULEINFO *)NULL;
	char *mod_name = (char *)NULL;
	char *mod_name_proc = (char *)NULL;

	proc_list = pinfo_getproclist ();
	if (proc_list != (PROC_LIST_T *)NULL && matcher != (char *)NULL) {

		pcre *regex = pinfo_pcre_compile (matcher);
		if (regex == (pcre *)NULL) {
			free (proc_list->list);
			free (proc_list);
			fprintf (stderr, "Can't compile matcher '%s'\n", matcher);
			return;
		}

		for (proc_runner = 0; proc_runner < proc_list->required_count; proc_runner++) {
			proc_current = proc_list->list[proc_runner];
			if (proc_current != 0) {
				proc_handle = OpenProcess (PROCESS_ALL_ACCESS, FALSE, proc_current);
				if (proc_handle != 0) {
					mod_list = pinfo_getprocmodlist (proc_handle);
					if (mod_list != (HMODULE_LIST_T *)NULL) {
						if (mod_list->list_count >= 1) {
							mod_current = mod_list->list[0];
							mod_name_proc = pinfo_getmodname (proc_handle, mod_current);
							mod_info = pinfo_getmodinfo (proc_handle, mod_current);
							if ((pinfo_pcre_match (regex, mod_name_proc)) == 1) {
								if (mod_name_proc != (char *)NULL && mod_info != (MODULEINFO *)NULL) {
									printf ("[%ld] %s\n", proc_current, mod_name_proc);
									printf ("\tBase Addr   : 0x%p\n", (void *)mod_info->lpBaseOfDll);
									printf ("\tEntry Point : 0x%p\n", (void *)mod_info->EntryPoint);
									print_unit (mod_info->SizeOfImage, units, 1);
								}
								if (mod_info != (MODULEINFO *)NULL) {
									free (mod_info);
								}
								if (longlist == 1) {
									printf ("\tModules:\n");
									for (mod_runner = 1; mod_runner < mod_list->list_count; mod_runner++) {
										mod_current = mod_list->list[mod_runner];
										mod_name = pinfo_getmodname (proc_handle, mod_current);
										if (mod_name != (char *)NULL) {
											printf ("\t%s\n", mod_name);
											free (mod_name);
										}
										mod_info = pinfo_getmodinfo (proc_handle, mod_current);
										if (mod_info != (MODULEINFO *)NULL) {
											printf ("\t\tBase Addr   : 0x%p\n", (void *)mod_info->lpBaseOfDll);
											printf ("\t\tEntry Point : 0x%p\n", (void *)mod_info->EntryPoint);
											print_unit (mod_info->SizeOfImage, units, 2);
											free (mod_info);
										}
									}
								}
								if ((pinfo_paction (proc_handle, proc_current, mod_name_proc, actionmode)) != 0) {
									fprintf (stderr, "All actions over %ld done!\n", proc_current);
								}
								if (mod_name_proc != (char *)NULL) {
									free (mod_name_proc);
								}
							}
							free (mod_list->list);
							free (mod_list);
						}
					}
				}
				CloseHandle (proc_handle);
			}
		}
		pinfo_pcre_free (regex);
		free (proc_list->list);
		free (proc_list);
	}
}


void
pinfo_msearch_list (const char *matcher, int longlist, int actionmode, int units) {
	PROC_LIST_T *proc_list = (PROC_LIST_T *)NULL;
	DWORD proc_current;
	DWORD proc_runner;
	HANDLE proc_handle;

	HMODULE_LIST_T *mod_list = (HMODULE_LIST_T *)NULL;
	DWORD mod_runner;
	HMODULE mod_current;
	HMODULE modules[2048];
	char *modnames[2048];
	int modules_pos = 0;
	int modules_count = 0;
	MODULEINFO *mod_info = (MODULEINFO *)NULL;
	char *mod_name;
	char *mod_name_proc;

	int matches = 0;

	proc_list = pinfo_getproclist ();
	if (proc_list != (PROC_LIST_T *)NULL && matcher != (char *)NULL) {

		pcre *regex = pinfo_pcre_compile (matcher);
		if (regex == (pcre *)NULL) {
			free (proc_list->list);
			free (proc_list);
			fprintf (stderr, "Can't compile matcher '%s'\n", matcher);
			return;
		}

		for (proc_runner = 0; proc_runner < proc_list->required_count; proc_runner++) {
			memset ((void *)modules, 0, (sizeof(HMODULE) * (size_t)1024));
			memset ((void *)modnames, 0, 2048);
			modules_pos = 0;
			proc_current = proc_list->list[proc_runner];
			if (proc_current != 0) {
				proc_handle = OpenProcess (PROCESS_ALL_ACCESS, FALSE, proc_current);
				if (proc_handle != 0) {
					mod_list = pinfo_getprocmodlist (proc_handle);
					if (mod_list != (HMODULE_LIST_T *)NULL) {
						if (mod_list->list_count >= 1) {
							mod_current = mod_list->list[0];
							for (mod_runner = 1; mod_runner < mod_list->list_count; mod_runner++) {
								mod_current = mod_list->list[mod_runner];
								mod_name = pinfo_getmodname (proc_handle, mod_current);
								if ((pinfo_pcre_match (regex, mod_name)) == 1) {
									modules[modules_pos] = mod_current;
									modnames[modules_pos] = strdup(mod_name);
									modules_pos++;
									matches = 1;
								} else {
									matches = 0;
								}
								if (mod_name != (char *)NULL) {
									free (mod_name);
								}
								if (matches == 1) {
									mod_current = mod_list->list[0];
									mod_name_proc = pinfo_getmodname (proc_handle, mod_current);
									mod_info = pinfo_getmodinfo (proc_handle, mod_current);
									if (mod_name != (char *)NULL && mod_info != (MODULEINFO *)NULL) {
										printf ("[%ld] %s\n", proc_current, mod_name);
										printf ("\tBase Addr   : 0x%p\n", (void *)mod_info->lpBaseOfDll);
										printf ("\tEntry Point : 0x%p\n", (void *)mod_info->EntryPoint);
										print_unit (mod_info->SizeOfImage, units, 1);
									}
									if (mod_info != (MODULEINFO *)NULL) {
										free (mod_info);
									}
									if (longlist == 1) {
										printf ("\tModules:\n");
										for (mod_runner = 1; mod_runner < mod_list->list_count; mod_runner++) {
											mod_current = mod_list->list[mod_runner];
											mod_name = pinfo_getmodname (proc_handle, mod_current);
											if (mod_name != (char *)NULL) {
												printf ("\t%s\n", mod_name);

												free (mod_name);
											}
											mod_info = pinfo_getmodinfo (proc_handle, mod_current);
											if (mod_info != (MODULEINFO *)NULL) {
												printf ("\t\tBase Addr   : 0x%p\n", (void *)mod_info->lpBaseOfDll);
												printf ("\t\tEntry Point : 0x%p\n", (void *)mod_info->EntryPoint);
												print_unit (mod_info->SizeOfImage, units, 2);
												free (mod_info);
											}
										}
									}
									for (modules_count = 0; modules_count < modules_pos && modules[modules_count] != 0; modules_count++) {
										if (modules[modules_count] == mod_current) {
											printf ("%s\n", modnames[modules_count]);
											if ((pinfo_maction (proc_handle, proc_current, mod_current, modnames[modules_count], actionmode)) != 0) {
												fprintf (stderr, "All actions over %ld done!\n", proc_current);
											}
										}
									}
									if ((pinfo_paction (proc_handle, proc_current, mod_name_proc, actionmode)) != 0) {
										fprintf (stderr, "All actions over %ld done!\n", proc_current);
									}
									if (mod_name_proc != (char *)NULL) {
										free (mod_name_proc);
									}
								}
							}
						}
						free (mod_list->list);
						free (mod_list);
					}
				}
				CloseHandle (proc_handle);
				for (modules_count = 0; modules_count < modules_pos && modules[modules_count] != 0; modules_count++) {
					if (modnames[modules_count] != (char *)NULL) {
						free (modnames[modules_count]);
					}
				}
			}
		}
		pinfo_pcre_free (regex);
		free (proc_list->list);
		free (proc_list);
	}
}
