/*
 procinfo -- list process information
 Copyright (C) 2007 Daniel Molina Wegener <dmw@coder.cl>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307  US
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define PCRE_STATIC 1

#include <pcre.h>

#include "pinfo_grep.h"

#define PCRE_LOCAL_OPTIONS			PCRE_CASELESS | PCRE_NEWLINE_ANY

static pcre_extra *local_study;
static char *local_regex;

pcre *
pinfo_pcre_compile (const char *regex) {

	char *regex_error = (char *)NULL;
	int regex_error_off = 0;

	pcre *r = (pcre *)NULL;
	if (regex != (const char *)NULL) {
		local_regex = (char *)regex;
		r = pcre_compile (regex, PCRE_LOCAL_OPTIONS, (const char **)&regex_error,
							&regex_error_off, (const unsigned char *)NULL);
		if (r != (pcre *)NULL) {
			local_study = pcre_study (r, 0, (const char **)&regex_error);
		} else {
			fprintf (stderr, "Error compiling '%s' [%s]\n", regex, regex_error);
		}
	}
	return r;
}


#define REGEX_PCRE_OV_SZ		2048

int
pinfo_pcre_match (const pcre *regex, const char *str) {
	int r = 0;
	int ov[REGEX_PCRE_OV_SZ];
	int x = 0;
	if (regex != (const pcre *)NULL && str != (const char *)NULL) {
		memset (ov, (int)NULL, REGEX_PCRE_OV_SZ);
		x = pcre_exec (regex, local_study, str, strlen (str), 0, 0, ov, REGEX_PCRE_OV_SZ);
		if (x > 0) {
			r = 1;
		}
	}
	return r;
}


void
pinfo_pcre_free (pcre *regex) {
	if (regex != (pcre *)NULL) {
		free (regex);
	}
	if (local_study != (pcre_extra *)NULL) {
		free (local_study);
	}
}



